@@ -20,6 +20,7 @@
 #include "kgrapheditor.h"
 #include "kgrapheditorConfigDialog.h"
 #include "kgrapheditorsettings.h"
+#include "kgrapheditor_debug.h"
 #include "KGraphEditorNodesTreeWidget.h"
 #include "KGraphEditorElementTreeWidget.h"
 #include "ui_preferencesReload.h"
@@ -29,25 +30,26 @@
 #include "part/kgraphviewer_part.h"
 
 #include <kshortcutsdialog.h>
-#include <kfiledialog.h>
+#include <QFileDialog>
 #include <kconfig.h>
-#include <kurl.h>
-#include <ktabwidget.h>
+#include <QUrl>
+#include <QTabWidget>
+#include <kparts/partmanager.h>
 #include <kedittoolbar.h>
-#include <kdebug.h>
-#include <kstandarddirs.h>
+#include <QDebug>
+#include <QStandardPaths>
 #include <kstandardaction.h>
 #include <ktoggleaction.h>
-#include <klibloader.h>
-#include <kmessagebox.h>
-#include <kstatusbar.h>
-#include <klocale.h>
+#include <KPluginLoader>
+#include <KService>
+#include <QMessageBox>
+#include <QStatusBar>
 #include <kconfigdialog.h>
-#include <kiconloader.h>
+#include <QDebug>
 #include <krecentfilesaction.h>
 #include <ktoolbar.h>
 #include <KActionCollection>
-
+#include <klocalizedstring.h>
 #include <QtDBus/QtDBus>
 #include <QDockWidget>
 #include <QTreeWidget>
@@ -57,21 +59,22 @@
 using namespace KGraphViewer;
 
 KGraphEditor::KGraphEditor() :
-    KXmlGuiWindow(), 
-    m_rfa(0),
-    m_currentPart(0)
+    KParts::MainWindow(),
+    m_rfa(nullptr),
+    m_currentPart(nullptr)
 {
   // set the shell's ui resource file
   setXMLFile("kgrapheditorui.rc");
 
-  m_widget = new KTabWidget(this);
-  m_widget->setHoverCloseButton(true);
-  connect(m_widget, SIGNAL(closeRequest(QWidget*)), this, SLOT(close(QWidget*)));
-  connect(m_widget, SIGNAL(currentChanged(QWidget*)), this, SLOT(newTabSelectedSlot(QWidget*)));
-  
+  m_widget = new QTabWidget(this);
+  m_widget->setTabsClosable(true);
+  connect(m_widget, SIGNAL(tabCloseRequested(int)), this, SLOT(close(int)));
+  connect(m_widget, SIGNAL(currentChanged(int)), this, SLOT(newTabSelectedSlot(int)));
+
   setCentralWidget(m_widget);
 
   QDockWidget* topLeftDockWidget = new QDockWidget(this);
+  topLeftDockWidget->setObjectName(QStringLiteral("TopLeftDockWidget"));
   m_treeWidget = new KGraphEditorNodesTreeWidget(topLeftDockWidget);
   connect(m_treeWidget,SIGNAL(itemChanged(QTreeWidgetItem*,int)),
           this,SLOT(slotItemChanged(QTreeWidgetItem*,int)));
@@ -90,6 +93,7 @@ KGraphEditor::KGraphEditor() :
   addDockWidget ( Qt::LeftDockWidgetArea, topLeftDockWidget );
 
   QDockWidget* bottomLeftDockWidget = new QDockWidget(this);
+  bottomLeftDockWidget->setObjectName(QStringLiteral("BottomLeftDockWidget"));
   m_newElementAttributesWidget = new KGraphEditorElementTreeWidget(bottomLeftDockWidget);
   connect(m_newElementAttributesWidget,SIGNAL(itemChanged(QTreeWidgetItem*,int)),
           this,SLOT(slotNewElementItemChanged(QTreeWidgetItem*,int)));
@@ -104,51 +108,46 @@ KGraphEditor::KGraphEditor() :
 
   if (QDBusConnection::sessionBus().registerService( "org.kde.kgrapheditor" ))
   {
-    kDebug() << "Service Registered successfully";
+    qCDebug(KGRAPHEDITOR_LOG) << "Service Registered successfully";
     QDBusConnection::sessionBus().registerObject("/", this, QDBusConnection::ExportAllSlots);
-    
+
   }
   else
   {
-    kDebug() << "Failed to register service...";
+    qCDebug(KGRAPHEDITOR_LOG) << "Failed to register service...";
   }
 
+  // Create a KParts part manager, to handle part activation/deactivation
+  m_manager = new KParts::PartManager( this );
+
+  // When the manager says the active part changes, the window updates (recreates) the GUI
+  connect( m_manager, SIGNAL(activePartChanged(KParts::Part*)),
+           this, SLOT(createGUI(KParts::Part*)) );
+
+  setupGUI(ToolBar | Keys | StatusBar | Save);
+
   // then, setup our actions
   setupActions();
 
-  // and a status bar
-  statusBar()->show();
-  createStandardStatusBarAction();
-
   // this routine will find and load our Part.  it finds the Part by
   // name which is a bad idea usually.. but it's alright in this
   // case since our Part is made for this Shell
 
   // Creates the GUI with a null part to make appear the main app menus and tools
   createGUI(0);
-  setupGUI();  
-  // apply the saved mainwindow settings, if any, and ask the mainwindow
-  // to automatically save settings if changed: window size, toolbar
-  // position, icon size, etc.
-  setAutoSaveSettings();
-
 }
 
 KGraphEditor::~KGraphEditor()
 {
-  kDebug() ;
 }
 
 void KGraphEditor::reloadPreviousFiles()
 {
   QStringList previouslyOpenedFiles = KGraphEditorSettings::previouslyOpenedFiles();
-  if ( (previouslyOpenedFiles.empty() == false) 
-       && (KMessageBox::questionYesNo(this, 
-              i18n("Do you want to reload files from previous session?"),
+  if ( (previouslyOpenedFiles.empty() == false)
+       && (QMessageBox::question(this,
               i18n("Session Restore"),
-              KStandardGuiItem::yes(),
-              KStandardGuiItem::no(),
-              "reopenPreviouslyOpenedFilesMode"   ) == KMessageBox::Yes) )
+              i18n("Do you want to reload files from previous session?")) == QMessageBox::Yes) )
   {
     QStringList::const_iterator it, it_end;
     it = previouslyOpenedFiles.constBegin(); it_end = previouslyOpenedFiles.constEnd();
@@ -157,50 +156,50 @@ void KGraphEditor::reloadPreviousFiles()
       openUrl(*it);
     }
   }
-  
 }
 
 KParts::ReadOnlyPart *KGraphEditor::slotNewGraph()
 {
-  kDebug();
   KPluginFactory *factory = KPluginLoader("kgraphviewerpart").factory();
   if (!factory)
   {
     // if we couldn't find our Part, we exit since the Shell by
     // itself can't do anything useful
-    KMessageBox::error(this, i18n("Could not find the KGraphViewer part."));
-    kapp->quit();
+    QMessageBox::critical(this, i18n("Unable to start"), i18n("Could not find the KGraphViewer part."));
+    qApp->quit();
     // we return here, cause kapp->quit() only means "exit the
     // next time we enter the event loop...
-    return NULL;
+    return nullptr;
   }
-  KParts::ReadOnlyPart* part = factory->create<KParts::ReadOnlyPart>("kgraphviewerpart", this);
+  KParts::ReadOnlyPart* part = factory->create<KParts::ReadOnlyPart>(this);
   KGraphViewerInterface *view = qobject_cast<KGraphViewerInterface *>(part);
   if (!view)
   {
     // This should not happen
-    kError() << "Failed to get KPart" << endl;
-    return NULL;
+    qCWarning(KGRAPHEDITOR_LOG) << "Failed to get KPart" << endl;
+    return nullptr;
   }
   view->setReadWrite();
 
     QWidget *w = part->widget();
-    m_widget->addTab(w, QIcon( DesktopIcon("kgraphviewer") ), "");
-    m_widget->setCurrentWidget(w);
-//     createGUI(view);
 
     m_tabsPartsMap[w] = part;
     m_tabsFilesMap[w] = "";
-//     connect(this,SIGNAL(hide(KParts::Part*)),view,SLOT(slotHide(KParts::Part*)));
-  slotSetActiveGraph(part);
+    connect(this,SIGNAL(hide(KParts::Part*)),part,SLOT(slotHide(KParts::Part*)));
+
+    m_manager->addPart(part, true);
+
+    m_widget->addTab(w, QIcon::fromTheme("kgraphviewer"), "");
+    m_widget->setCurrentWidget(w);
+    m_closeAction->setEnabled(true);
   return part;
 }
 
-void KGraphEditor::openUrl(const KUrl& url)
+void KGraphEditor::openUrl(const QUrl& url)
 {
-  kDebug() << url;
+  qCDebug(KGRAPHEDITOR_LOG) << url;
   KParts::ReadOnlyPart *part = slotNewGraph();
-  
+
 //   (KGraphEditorSettings::parsingMode() == "external")
 //     ?kgv->setLayoutMethod(KGraphViewerInterface::ExternalProgram)
 //     :kgv->setLayoutMethod(KGraphViewerInterface::InternalLibrary);
@@ -211,27 +210,28 @@ void KGraphEditor::openUrl(const KUrl& url)
   m_tabsFilesMap[m_widget->currentWidget()] = url.path();
   part->openUrl(url);
 
+  if (m_rfa) {
+    m_rfa->addUrl(url);
+  }
+
   m_openedFiles.push_back(url.path());
 }
 
 void KGraphEditor::fileOpen()
 {
-  kDebug() ;
   // this slot is called whenever the File->Open menu is selected,
   // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
   // button is clicked
-  QStringList file_names = KFileDialog::getOpenFileNames(KUrl(QString()), QString("*.dot"), 0, QString());
-  
-  if (!file_names.empty())
-  {
-    foreach (const QString &fileName, file_names)
-    {
-      if (m_rfa != 0)
-      {
-        m_rfa->addUrl(fileName);
-      }
-      openUrl(fileName);
-    }
+  QFileDialog fileDialog(this);
+  fileDialog.setMimeTypeFilters(QStringList(QStringLiteral("text/vnd.graphviz")));
+  fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+  fileDialog.setFileMode(QFileDialog::ExistingFiles);
+  if (fileDialog.exec() != QFileDialog::Accepted) {
+    return;
+  }
+
+  foreach (const QUrl& url, fileDialog.selectedUrls()) {
+    openUrl(url);
   }
 }
 
@@ -242,12 +242,16 @@ void KGraphEditor::setupActions()
 
   actionCollection()->addAction( KStandardAction::New, "file_new", this, SLOT(fileNew()) );
   actionCollection()->addAction( KStandardAction::Open, "file_open", this, SLOT(fileOpen()) );
-  m_rfa = (KRecentFilesAction*) actionCollection()->addAction(KStandardAction::OpenRecent, "file_open_recent", this, SLOT(slotURLSelected(KUrl)) );
-  m_rfa->loadEntries(KGlobal::config()->group("kgrapheditor"));
+  m_rfa = (KRecentFilesAction*) actionCollection()->addAction(KStandardAction::OpenRecent, "file_open_recent", this, SLOT(slotURLSelected(QUrl)) );
+  m_rfa->loadEntries(KConfigGroup(KSharedConfig::openConfig(), "kgrapheditor"));
   actionCollection()->addAction( KStandardAction::Save, "file_save", this, SLOT(fileSave()) );
   actionCollection()->addAction( KStandardAction::SaveAs, "file_save_as", this, SLOT(fileSaveAs()) );
 
-  actionCollection()->addAction( KStandardAction::Quit, "file_quit", KApplication::kApplication(), SLOT(quit()) );
+  m_closeAction = actionCollection()->addAction( KStandardAction::Close, "file_close", this, SLOT(close()) );
+  m_closeAction->setWhatsThis(i18n("Closes the current file"));
+  m_closeAction->setEnabled(false);
+
+  actionCollection()->addAction( KStandardAction::Quit, "file_quit", qApp, SLOT(quit()) );
 
   m_statusbarAction = KStandardAction::showStatusbar(this, SLOT(optionsShowStatusbar()), this);
 
@@ -256,24 +260,23 @@ void KGraphEditor::setupActions()
   actionCollection()->addAction( KStandardAction::ConfigureToolbars, "options_configure_toolbars", this, SLOT(optionsConfigureToolbars()) );
   actionCollection()->addAction( KStandardAction::Preferences, "options_configure", this, SLOT(optionsConfigure()) );
 
-  KAction* edit_new_vertex = actionCollection()->addAction( "edit_new_vertex" );
+  QAction* edit_new_vertex = actionCollection()->addAction( "edit_new_vertex" );
   edit_new_vertex->setText(i18n("Create a New Vertex"));
-  edit_new_vertex->setIcon(QPixmap(KGlobal::dirs()->findResource("data","kgraphviewerpart/pics/kgraphviewer-newnode.png")));
+  edit_new_vertex->setIcon(QPixmap(QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kgraphviewerpart/pics/kgraphviewer-newnode.png")));
   connect( edit_new_vertex, SIGNAL(triggered(bool)), this, SLOT(slotEditNewVertex()) );
 
-  KAction* edit_new_edge = actionCollection()->addAction( "edit_new_edge" );
+  QAction* edit_new_edge = actionCollection()->addAction( "edit_new_edge" );
   edit_new_edge->setText(i18n("Create a New Edge"));
-  edit_new_edge->setIcon(QPixmap(KGlobal::dirs()->findResource("data","kgraphviewerpart/pics/kgraphviewer-newedge.png")));
+  edit_new_edge->setIcon(QPixmap(QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kgraphviewerpart/pics/kgraphviewer-newedge.png")));
   connect( edit_new_edge, SIGNAL(triggered(bool)), this, SLOT(slotEditNewEdge()) );
 }
 
 void KGraphEditor::closeEvent(QCloseEvent *event)
 {
-  kDebug() ;
   KGraphEditorSettings::setPreviouslyOpenedFiles(m_openedFiles);
-  m_rfa->saveEntries(KGlobal::config()->group("kgrapheditor"));
+  m_rfa->saveEntries(KConfigGroup(KSharedConfig::openConfig(), "kgrapheditor"));
 
-  KGraphEditorSettings::self()->writeConfig();
+  KGraphEditorSettings::self()->save();
   KXmlGuiWindow::closeEvent(event);
 }
 
@@ -314,7 +317,8 @@ void KGraphEditor::optionsConfigureKeys()
 
 void KGraphEditor::optionsConfigureToolbars()
 {
-    saveMainWindowSettings(KGlobal::config()->group("kgrapheditor") );
+    KConfigGroup conf(KConfigGroup(KSharedConfig::openConfig(), "kgrapheditor"));
+    saveMainWindowSettings(conf);
 
   // use the standard toolbar editor
   KEditToolBar dlg(factory());
@@ -325,19 +329,17 @@ void KGraphEditor::optionsConfigureToolbars()
 
 void KGraphEditor::optionsConfigure()
 {
-  //An instance of your dialog could be already created and could be cached, 
-  //in which case you want to display the cached dialog instead of creating 
-  //another one 
+  //An instance of your dialog could be already created and could be cached,
+  //in which case you want to display the cached dialog instead of creating
+  //another one
   if ( KgeConfigurationDialog::showDialog( "settings" ) )
-    return; 
- 
-  //KConfigDialog didn't find an instance of this dialog, so lets create it : 
-  KPageDialog::FaceType ft = KPageDialog::Auto;
-  KgeConfigurationDialog* dialog = new KgeConfigurationDialog( this, "settings",
-                                             KGraphEditorSettings::self(),ft );
+    return;
+
+  //KConfigDialog didn't find an instance of this dialog, so lets create it :
+  KgeConfigurationDialog* dialog = new KgeConfigurationDialog(this, "settings", KGraphEditorSettings::self());
 
   Ui::KGraphViewerPreferencesParsingWidget*  parsingWidget = dialog->m_parsingWidget;
-  kDebug() << KGraphEditorSettings::parsingMode();
+  qCDebug(KGRAPHEDITOR_LOG) << KGraphEditorSettings::parsingMode();
   if (KGraphEditorSettings::parsingMode() == "external")
   {
     parsingWidget->external->setChecked(true);
@@ -349,8 +351,8 @@ void KGraphEditor::optionsConfigure()
   connect((QObject*)parsingWidget->external, SIGNAL(toggled(bool)), this, SLOT(slotParsingModeExternalToggled(bool)) );
   connect((QObject*)parsingWidget->internal, SIGNAL(toggled(bool)), this, SLOT(slotParsingModeInternalToggled(bool)) );
 
-                                             
-/*  KGraphViewerPreferencesReloadWidget*  reloadWidget =  
+
+/*  KGraphViewerPreferencesReloadWidget*  reloadWidget =
       new KGraphViewerPreferencesReloadWidget( 0, "KGraphViewer Settings" );
   if (KGraphEditorSettings::reloadOnChangeMode() == "yes")
   {
@@ -364,12 +366,12 @@ void KGraphEditor::optionsConfigure()
   {
     reloadWidget->reloadOnChangeMode->setButton(2);
   }
-  
+
   connect((QObject*)reloadWidget->reloadOnChangeMode, SIGNAL(clicked(int)), this, SLOT(reloadOnChangeMode_pressed(int)) );
-  
-  dialog->addPage( reloadWidget, i18n("Reloading"), "kgraphreloadoptions", i18n("Reloading"), true); 
- 
-  KGraphViewerPreferencesOpenInExistingWindowWidget*  openingWidget =  
+
+  dialog->addPage( reloadWidget, i18n("Reloading"), "kgraphreloadoptions", i18n("Reloading"), true);
+
+  KGraphViewerPreferencesOpenInExistingWindowWidget*  openingWidget =
     new KGraphViewerPreferencesOpenInExistingWindowWidget( 0, "KGraphViewer Settings" );
   if (KGraphEditorSettings::openInExistingWindowMode() == "yes")
   {
@@ -383,12 +385,12 @@ void KGraphEditor::optionsConfigure()
   {
     openingWidget->openInExistingWindowMode->setButton(2);
   }
-  
+
   connect((QObject*)openingWidget->openInExistingWindowMode, SIGNAL(clicked(int)), this, SLOT(openInExistingWindowMode_pressed(int)) );
-  
-  dialog->addPage( openingWidget, i18n("Opening"), "kgraphopeningoptions", i18n("Opening"), true); 
-  
-  KGraphViewerPreferencesReopenPreviouslyOpenedFilesWidget*  reopeningWidget =  
+
+  dialog->addPage( openingWidget, i18n("Opening"), "kgraphopeningoptions", i18n("Opening"), true);
+
+  KGraphViewerPreferencesReopenPreviouslyOpenedFilesWidget*  reopeningWidget =
     new KGraphViewerPreferencesReopenPreviouslyOpenedFilesWidget( 0, "KGraphViewer Settings" );
   if (KGraphEditorSettings::reopenPreviouslyOpenedFilesMode() == "yes")
   {
@@ -402,10 +404,10 @@ void KGraphEditor::optionsConfigure()
   {
     reopeningWidget->reopenPreviouslyOpenedFilesMode->setButton(2);
   }
-  
+
   connect((QObject*)reopeningWidget->reopenPreviouslyOpenedFilesMode, SIGNAL(clicked(int)), this, SLOT(reopenPreviouslyOpenedFilesMode_pressed(int)) );
 
-  dialog->addPage( reopeningWidget, i18n("Session Management"), "kgraphreopeningoptions", i18n("Session Management"), true); 
+  dialog->addPage( reopeningWidget, i18n("Session Management"), "kgraphreopeningoptions", i18n("Session Management"), true);
   */
 //   connect(dialog, SIGNAL(settingsChanged()), this, SLOT(settingsChanged()));
 
@@ -414,12 +416,12 @@ void KGraphEditor::optionsConfigure()
 
 void KGraphEditor::applyNewToolbarConfig()
 {
-  applyMainWindowSettings(KGlobal::config()->group("kgrapheditor"));
+  applyMainWindowSettings(KConfigGroup(KSharedConfig::openConfig(), "kgrapheditor"));
 }
 
 // void KGraphViewer::reloadOnChangeMode_pressed(int value)
 // {
-//   kDebug() << "reloadOnChangeMode_pressed " << value;
+//   qCDebug(KGRAPHEDITOR_LOG) << "reloadOnChangeMode_pressed " << value;
 //   switch (value)
 //   {
 //   case 0:
@@ -432,14 +434,14 @@ void KGraphEditor::applyNewToolbarConfig()
 //     KGraphEditorSettings::setReloadOnChangeMode("ask");
 //     break;
 //   default:
-//   kError() << "Invalid reload on change mode value: " << value;
+//   qCWarning(KGRAPHEDITOR_LOG) << "Invalid reload on change mode value: " << value;
 //     return;
 //   }
-//   kDebug() << "emiting";
+//   qCDebug(KGRAPHEDITOR_LOG) << "emiting";
 //   emit(settingsChanged());
-//   KGraphEditorSettings::writeConfig();
+//   KGraphEditorSettings::save();
 // }
-// 
+//
 // void KGraphViewer::openInExistingWindowMode_pressed(int value)
 // {
 //   std::cerr << "openInExistingWindowMode_pressed " << value << std::endl;
@@ -455,15 +457,15 @@ void KGraphEditor::applyNewToolbarConfig()
 //     KGraphEditorSettings::setOpenInExistingWindowMode("ask");
 //     break;
 //   default:
-//   kError() << "Invalid OpenInExistingWindow value: " << value << endl;
+//   qCWarning(KGRAPHEDITOR_LOG) << "Invalid OpenInExistingWindow value: " << value << endl;
 //     return;
 //   }
-// 
+//
 //   std::cerr << "emiting" << std::endl;
 //   emit(settingsChanged());
-//   KGraphEditorSettings::writeConfig();
+//   KGraphEditorSettings::save();
 // }
-// 
+//
 // void KGraphViewer::reopenPreviouslyOpenedFilesMode_pressed(int value)
 // {
 //   std::cerr << "reopenPreviouslyOpenedFilesMode_pressed " << value << std::endl;
@@ -479,38 +481,41 @@ void KGraphEditor::applyNewToolbarConfig()
 //     KGraphEditorSettings::setReopenPreviouslyOpenedFilesMode("ask");
 //     break;
 //   default:
-//   kError() << "Invalid ReopenPreviouslyOpenedFilesMode value: " << value << endl;
+//   qCWarning(KGRAPHEDITOR_LOG) << "Invalid ReopenPreviouslyOpenedFilesMode value: " << value << endl;
 //     return;
 //   }
-// 
+//
 //   std::cerr << "emiting" << std::endl;
 //   emit(settingsChanged());
-//   KGraphEditorSettings::writeConfig();
+//   KGraphEditorSettings::save();
 // }
 
 
-void KGraphEditor::slotURLSelected(const KUrl& url)
+void KGraphEditor::slotURLSelected(const QUrl& url)
 {
   openUrl(url);
 }
 
-void KGraphEditor::close(QWidget* tab)
+void KGraphEditor::close(int index)
 {
+  QWidget *tab = m_widget->widget(index);
   m_openedFiles.removeAll(m_tabsFilesMap[tab]);
-  m_widget->removePage(tab);
+  m_widget->removeTab(index);
   tab->hide();
   KParts::ReadOnlyPart *part = m_tabsPartsMap[tab];
+  m_manager->removePart(part);
   m_tabsPartsMap.remove(tab);
   m_tabsFilesMap.remove(tab);
-  delete part; part=0;
+  delete part; part = nullptr;
 /*  delete tab;
-  tab = 0;*/
+  tab = nullptr;*/
+  m_closeAction->setEnabled(m_widget->count() > 0);
 }
 
 void KGraphEditor::close()
 {
-  QWidget* currentPage = m_widget->currentWidget();
-  if (currentPage != 0)
+  int currentPage = m_widget->currentIndex();
+  if (currentPage != -1)
   {
     close(currentPage);
   }
@@ -519,7 +524,7 @@ void KGraphEditor::close()
 void KGraphEditor::fileSave()
 {
   QWidget* currentPage = m_widget->currentWidget();
-  if (currentPage != 0)
+  if (currentPage)
   {
     emit(saveTo(QUrl(m_tabsFilesMap[currentPage]).path()));
   }
@@ -528,31 +533,38 @@ void KGraphEditor::fileSave()
 void KGraphEditor::fileSaveAs()
 {
   QWidget* currentPage = m_widget->currentWidget();
-  if (currentPage != 0)
+  if (currentPage)
   {
-    QString fileName = KFileDialog::getSaveFileName(KUrl(),
-                "*.dot", currentPage,
-                i18n("Save current graph to..."));
+    QFileDialog fileDialog(currentPage, i18n("Save current graph"));
+    fileDialog.setMimeTypeFilters(QStringList(QStringLiteral("text/vnd.graphviz")));
+    fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+    if (fileDialog.exec() != QFileDialog::Accepted) {
+      return;
+    }
+    const QString fileName = fileDialog.selectedFiles().at(0);
+    if (fileName.isEmpty()) {
+      return;
+    }
+
     m_tabsFilesMap[currentPage] = fileName;
     emit(saveTo(fileName));
   }
 }
 
-void KGraphEditor::newTabSelectedSlot(QWidget* tab)
+void KGraphEditor::newTabSelectedSlot(int index)
 {
-//   kDebug() << tab;
-//   emit(hide((KParts::Part*)(m_manager->activePart())));
-  if (!m_tabsPartsMap.isEmpty())
-  {
+//   qCDebug(KGRAPHEDITOR_LOG) << tab;
+  emit(hide((KParts::Part*)(m_manager->activePart())));
+  QWidget *tab = m_widget->widget(index);
+  if (tab) {
     slotSetActiveGraph(m_tabsPartsMap[tab]);
-//     m_manager->setActivePart(m_tabsPartsMap[tab]);
+    m_manager->setActivePart(m_tabsPartsMap[tab]);
   }
 }
 
 void KGraphEditor::slotSetActiveGraph(KParts::ReadOnlyPart *part)
 {
-  kDebug();
-  if (m_currentPart != 0)
+  if (m_currentPart)
   {
     disconnect(this,SIGNAL(prepareAddNewElement(QMap<QString,QString>)),m_currentPart,SLOT(prepareAddNewElement(QMap<QString,QString>)));
     disconnect(this,SIGNAL(prepareAddNewEdge(QMap<QString,QString>)),m_currentPart,SLOT(prepareAddNewEdge(QMap<QString,QString>)));
@@ -570,7 +582,7 @@ void KGraphEditor::slotSetActiveGraph(KParts::ReadOnlyPart *part)
   }
   m_currentPart = part;
   m_treeWidget->clear();
-  if (m_currentPart == 0)
+  if (m_currentPart == nullptr)
   {
     return;
   }
@@ -586,13 +598,13 @@ void KGraphEditor::slotSetActiveGraph(KParts::ReadOnlyPart *part)
   connect(this,SIGNAL(saddNewEdge(QString,QString,QMap<QString,QString>)),m_currentPart,SLOT(slotAddNewEdge(QString,QString,QMap<QString,QString>)));
   connect(this,SIGNAL(renameNode(QString,QString)),m_currentPart,SLOT(slotRenameNode(QString,QString)));
   connect(this,SIGNAL(setAttribute(QString,QString,QString)),m_currentPart,SLOT(slotSetAttribute(QString,QString,QString)));
-  
+
   QList<QString> nodesIds;//TODO = m_currentPart->nodesIds();
   QList<QTreeWidgetItem *> items;
   foreach (const QString& nodeId, nodesIds)
   {
-    kDebug()<< "new item " << nodeId;
-    QTreeWidgetItem* item = new QTreeWidgetItem((QTreeWidget*)0, QStringList(nodeId));
+    qCDebug(KGRAPHEDITOR_LOG)<< "new item " << nodeId;
+    QTreeWidgetItem* item = new QTreeWidgetItem((QTreeWidget*)nullptr, QStringList(nodeId));
     item->setFlags(item->flags() | Qt::ItemIsEditable);
     QMap<QString,QString> attributes;//TODO = m_currentPart->nodeAtributes(nodeId);
     foreach (const QString& attrib, attributes.keys())
@@ -601,14 +613,14 @@ void KGraphEditor::slotSetActiveGraph(KParts::ReadOnlyPart *part)
       {
         QStringList list(attrib);
         list << attributes[attrib];
-        QTreeWidgetItem* child = new QTreeWidgetItem((QTreeWidget*)0, list);
+        QTreeWidgetItem* child = new QTreeWidgetItem((QTreeWidget*)nullptr, list);
         child->setFlags(child->flags() | Qt::ItemIsEditable);
         item->addChild(child);
       }
     }
     items.append(item);
   }
-  kDebug() << "inserting";
+  qCDebug(KGRAPHEDITOR_LOG) << "inserting";
   m_treeWidget->insertTopLevelItems(0, items);
 
 
@@ -632,33 +644,33 @@ void KGraphEditor::slotSetActiveGraph(KParts::ReadOnlyPart *part)
 
   connect( m_currentPart, SIGNAL(hoverEnter(QString)),
            this, SLOT(slotHoverEnter(QString)) );
-            
+
   connect( m_currentPart, SIGNAL(hoverLeave(QString)),
           this, SLOT(slotHoverLeave(QString)) );
 }
 
 void KGraphEditor::slotNewNodeAdded(const QString& id)
 {
-  kDebug() << id;
+  qCDebug(KGRAPHEDITOR_LOG) << id;
   update();
 }
 
 void KGraphEditor::slotNewEdgeAdded(const QString& ids, const QString& idt)
 {
-  kDebug() << ids << idt;
+  qCDebug(KGRAPHEDITOR_LOG) << ids << idt;
   update();
 }
 
 void KGraphEditor::slotNewEdgeFinished( const QString& srcId, const QString& tgtId, const QMap<QString, QString>&attribs)
 {
-  kDebug() << srcId << tgtId << attribs;
+  qCDebug(KGRAPHEDITOR_LOG) << srcId << tgtId << attribs;
   emit saddNewEdge(srcId,tgtId,attribs);
   update();
 }
 
 void KGraphEditor::slotGraphLoaded()
 {
-  kDebug();
+  qCDebug(KGRAPHEDITOR_LOG);
   disconnect(m_treeWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)),
            this,SLOT(slotItemChanged(QTreeWidgetItem*,int)));
   disconnect(m_treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*,int)),
@@ -668,12 +680,12 @@ void KGraphEditor::slotGraphLoaded()
   QList<QTreeWidgetItem *> items;
   foreach (const QString& nodeId, nodesIds)
   {
-    kDebug()<< "item " << nodeId;
+    qCDebug(KGRAPHEDITOR_LOG)<< "item " << nodeId;
     QTreeWidgetItem* item;
     QList<QTreeWidgetItem*> existingItems = m_treeWidget->findItems(nodeId,Qt::MatchRecursive|Qt::MatchExactly);
     if (existingItems.isEmpty())
     {
-      item = new QTreeWidgetItem((QTreeWidget*)0, QStringList(nodeId));
+      item = new QTreeWidgetItem((QTreeWidget*)nullptr, QStringList(nodeId));
       items.append(item);
     }
     else
@@ -697,13 +709,13 @@ void KGraphEditor::slotGraphLoaded()
       {
         QStringList list(attrib);
         list << attributes[attrib];
-        QTreeWidgetItem* child = new QTreeWidgetItem((QTreeWidget*)0, list);
+        QTreeWidgetItem* child = new QTreeWidgetItem((QTreeWidget*)nullptr, list);
         child->setFlags(child->flags() | Qt::ItemIsEditable);
         item->addChild(child);
       }
     }
 }
-  kDebug() << "inserting";
+  qCDebug(KGRAPHEDITOR_LOG) << "inserting";
   m_treeWidget->insertTopLevelItems(0, items);
   connect(m_treeWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)),
            this,SLOT(slotItemChanged(QTreeWidgetItem*,int)));
@@ -713,7 +725,7 @@ void KGraphEditor::slotGraphLoaded()
 
 void KGraphEditor::slotItemChanged ( QTreeWidgetItem * item, int column )
 {
-  kDebug() ;
+  qCDebug(KGRAPHEDITOR_LOG) ;
   /* values column */
   if (column == 0)
   {
@@ -724,7 +736,7 @@ void KGraphEditor::slotItemChanged ( QTreeWidgetItem * item, int column )
   else if (column == 1)
   {
     /* there is a parent ; it is an attribute line */
-    if (item->parent() != 0)
+    if (item->parent())
     {
       QString nodeLabel = item->parent()->text(0);
       QString attributeName = item->text(0);
@@ -737,10 +749,10 @@ void KGraphEditor::slotItemChanged ( QTreeWidgetItem * item, int column )
 
 void KGraphEditor::slotItemClicked ( QTreeWidgetItem * item, int column )
 {
-  kDebug() << column;
+  qCDebug(KGRAPHEDITOR_LOG) << column;
   m_currentTreeWidgetItemText = item->text(0);
 
-  QString nodeName = item->parent() != 0 ?
+  QString nodeName = item->parent() ?
                         item->parent()->text(0) :
                         item->text(0);
   emit selectNode(nodeName);
@@ -748,23 +760,23 @@ void KGraphEditor::slotItemClicked ( QTreeWidgetItem * item, int column )
 
 void KGraphEditor::slotEditNewVertex()
 {
-  if (m_currentPart == 0)
+  if (m_currentPart == nullptr)
   {
-    kDebug() << "new vertex: no part selected";
+    qCDebug(KGRAPHEDITOR_LOG) << "new vertex: no part selected";
     return;
   }
-  kDebug() << "new vertex";
+  qCDebug(KGRAPHEDITOR_LOG) << "new vertex";
   emit(prepareAddNewElement(m_newElementAttributes));
 }
 
 void KGraphEditor::slotEditNewEdge()
 {
-  if (m_currentPart == 0)
+  if (m_currentPart == nullptr)
   {
-    kDebug() << "new edge: no part selected";
+    qCDebug(KGRAPHEDITOR_LOG) << "new edge: no part selected";
     return;
   }
-  kDebug() << "new edge";
+  qCDebug(KGRAPHEDITOR_LOG) << "new edge";
   emit(prepareAddNewEdge(m_newElementAttributes));
 }
 
@@ -782,17 +794,16 @@ void KGraphEditor::slotAddAttribute(const QString& attribName)
 
 void KGraphEditor::slotRemoveAttribute(const QString& nodeName, const QString& attribName)
 {
-  kDebug();
   emit removeAttribute(nodeName,attribName);
   emit update();
 }
 
 void KGraphEditor::slotNewElementItemChanged(QTreeWidgetItem* item ,int column)
 {
-  kDebug();
+  qCDebug(KGRAPHEDITOR_LOG);
   if (column == 0)
   {
-    kError() << "Item id change not handled";
+    qCWarning(KGRAPHEDITOR_LOG) << "Item id change not handled";
     return;
   }
   else if (column == 1)
@@ -801,33 +812,31 @@ void KGraphEditor::slotNewElementItemChanged(QTreeWidgetItem* item ,int column)
   }
   else
   {
-    kError() << "Unknown column" << column;
+    qCWarning(KGRAPHEDITOR_LOG) << "Unknown column" << column;
     return;
   }
 }
 
 void KGraphEditor::slotAddNewElementAttribute(const QString& attrib)
 {
-  kDebug();
   m_newElementAttributes[attrib] = QString();
 }
 
 void KGraphEditor::slotRemoveNewElementAttribute(const QString& attrib)
 {
-  kDebug();
   m_newElementAttributes.remove(attrib);
 }
 
 void KGraphEditor::slotRemoveElement(const QString& id)
 {
-  kDebug() << id;
+  qCDebug(KGRAPHEDITOR_LOG) << id;
   m_treeWidget->slotRemoveElement(id);
   emit(removeElement(id));
 }
 
 void KGraphEditor::slotSelectionIs(const QList<QString>& elements , const QPoint& p)
 {
-  kDebug();
+  qCDebug(KGRAPHEDITOR_LOG);
   Q_UNUSED(p);
   QList<QTreeWidgetItem*> items = m_treeWidget->selectedItems();
   foreach (QTreeWidgetItem* item, items)
@@ -846,39 +855,34 @@ void KGraphEditor::slotSelectionIs(const QList<QString>& elements , const QPoint
 
 void KGraphEditor::slotParsingModeExternalToggled(bool value)
 {
-  kDebug();
   if (value)
   {
     KGraphEditorSettings::setParsingMode("external");
   }
-  //   kDebug() << "emiting";
+  //   qCDebug(KGRAPHEDITOR_LOG) << "emiting";
   //   emit(settingsChanged());
-  KGraphEditorSettings::self()->writeConfig();
+  KGraphEditorSettings::self()->save();
 }
 
 void KGraphEditor::slotParsingModeInternalToggled(bool value)
 {
-  kDebug();
   if (value)
   {
     KGraphEditorSettings::setParsingMode("internal");
   }
-  //   kDebug() << "emiting";
+  //   qCDebug(KGRAPHEDITOR_LOG) << "emiting";
   //   emit(settingsChanged());
-  KGraphEditorSettings::self()->writeConfig();
+  KGraphEditorSettings::self()->save();
 }
 
 void KGraphEditor::slotHoverEnter(const QString& id)
 {
-  kDebug() << id;
+  qCDebug(KGRAPHEDITOR_LOG) << id;
   statusBar()->showMessage(id);
 }
 
 void KGraphEditor::slotHoverLeave(const QString& id)
 {
-  kDebug() << id;
+  qCDebug(KGRAPHEDITOR_LOG) << id;
   statusBar()->showMessage("");
 }
-
-
-#include "kgrapheditor.moc"
