@@ -22,11 +22,11 @@
 
 //#include <config-kgraphviewer.h>
 
-#include <kapplication.h>
-#include <kparts/mainwindow.h>
-#include <ktabwidget.h>
-#include <kaction.h>
-#include <krecentfilesaction.h>
+#include <QAction>
+#include <QDir>
+#include <QTabWidget>
+#include <KParts/MainWindow>
+#include <KRecentFilesAction>
 
 #include <graphviz/gvc.h>
 
@@ -51,28 +51,30 @@ public:
   /**
     * Default Destructor
     */
-  virtual ~KGraphViewerWindow();
+  ~KGraphViewerWindow() override;
 
   /**
     * Use this method to load whatever file/URL you have
     */
-  void openUrl(const KUrl& url);
+  void openUrl(const QUrl& url);
 
   void reloadPreviousFiles();
 
 protected:
 
-  void closeEvent(QCloseEvent *event);
+  void closeEvent(QCloseEvent *event) override;
 
-signals:
+Q_SIGNALS:
   void hide(KParts::Part* part);
-  
-public slots:
+
+public Q_SLOTS:
   /**
     * Use this method to load whatever file/URL you have
     */
-  void openUrl(const QString& url) {openUrl(KUrl(url));}
-  
+  void openUrl(const QString& url) {
+      openUrl(QUrl::fromUserInput(url, QDir::currentPath(), QUrl::AssumeLocalFile));
+  }
+
   void close();
 
   void slotReloadOnChangeModeYesToggled(bool value);
@@ -86,39 +88,40 @@ public slots:
   void slotReopenPreviouslyOpenedFilesModeAskToggled(bool value);
   void slotParsingModeExternalToggled(bool value);
   void slotParsingModeInternalToggled(bool value);
-  
-private slots:
+
+private Q_SLOTS:
   void fileNew();
   void fileOpen();
-  void close(QWidget* tab);
-  void slotURLSelected(const KUrl&);
+  void close(int index);
+  void slotURLSelected(const QUrl&);
   void optionsShowToolbar();
   void optionsShowStatusbar();
   void optionsConfigureKeys();
   void optionsConfigureToolbars();
   void optionsConfigure();
-  void newTabSelectedSlot(QWidget* tab);
+  void newTabSelectedSlot(int index);
 
   void applyNewToolbarConfig();
 
   void slotHoverEnter(const QString&);
   void slotHoverLeave(const QString&);
   void slotBackgroundColorChanged(const QColor&);
-  
+
 private:
   void setupAccel();
   void setupActions();
-    
+
 private:
-  KTabWidget* m_widget;
+  QTabWidget* m_widget;
   KRecentFilesAction* m_rfa;
   KParts::PartManager* m_manager;
-  
+
   KToggleAction *m_toolbarAction;
   KToggleAction *m_statusbarAction;
+  QAction *m_closeAction;
 
   QStringList m_openedFiles;
-  
+
   QMap<QWidget*, KParts::Part*> m_tabsPartsMap;
   QMap<QWidget*, QString> m_tabsFilesMap;
 };
