@@ -28,6 +28,7 @@
 
 class LoadAGraphThread : public QThread
 {
+  Q_OBJECT
 public:
   LoadAGraphThread() : sem(1) {}
   void loadFile(const QString& dotFileName);
@@ -36,7 +37,7 @@ public:
   void processed_finished() { sem.release(); }
   
 protected:
-  virtual void run();
+  void run() override;
 
 private:
   QSemaphore sem;
